﻿define([
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/dom-class",
    "dojo/dom-geometry",
    "dojo/on",
    "dojo/when",
    "dojo/dom-construct",

// dijit
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/form/Button",
    "dijit/layout/_LayoutWidget",

// dgrid
    "dgrid/Keyboard",
    "dgrid/OnDemandList",
    "dgrid/Selection",

// epi
    "epi/dependency",
    "epi/shell/command/_WidgetCommandBinderMixin",
    "epi/shell/command/builder/MenuAssembler",
    "epi/shell/command/builder/ButtonBuilder",
    "epi/shell/dnd/Target",
    "epi/shell/dgrid/Formatter",
    "epi/shell/widget/_ModelBindingMixin",
    "epi/shell/widget/ContextMenu",
    "epi-cms/extension/events",
    "epi-cms/dgrid/formatters",
    "epi-cms/dgrid/WithContextMenu",
    "./ProjectComponentViewModel",
    "epi-cms/contentediting/ContentActionSupport",

// template
    "dojo/text!./ProjectComponent.html",

// Resources
    "epi/i18n!epi/cms/nls/episerver.cms.components.project",
    "epi/i18n!epi/nls/episerver.shared",

// Widgets in template
    "dijit/DropDownMenu",
    "dijit/form/DropDownButton",
    "dijit/Toolbar",
    "epi-cms/project/ProjectSelector"
],

function (
// dojo
    declare,
    lang,
    aspect,
    domClass,
    domGeometry,
    on,
    when,
    domConstruct,
// dijit
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    Button,
    _LayoutWidget,

// dgrid
    Keyboard,
    OnDemandList,
    Selection,

// epi
    dependency,
    _WidgetCommandBinderMixin,
    MenuAssembler,
    ButtonBuilder,
    Target,
    Formatter,
    _ModelBindingMixin,
    ContextMenu,
    events,
    formatters,
    WithContextMenu,
    ProjectComponentViewModel,
    ContentActionSupport,

// template
    template,

// Resources
    res,
    sharedResources
) {

    return declare([_LayoutWidget, _TemplatedMixin, _WidgetsInTemplateMixin, _ModelBindingMixin, _WidgetCommandBinderMixin], {
        // summary:
        //
        // tags:
        //      internal

        _listClass: declare([OnDemandList, Formatter, Selection, Keyboard, WithContextMenu]),

        _list: null,

        _dndTarget: null,

        _placeholderStatus: null,

        model: null,

        commandProvider: null,

        templateString: template,

        res: res,

        modelBindingMap: {
            "projectItemQuery": ["projectItemQuery"],
            "projectItemSortOrder": ["projectItemSortOrder"],
            "selectedProject": ["selectedProject"],
            "created": ["created"],
            "createdBy": ["createdBy"],
            "createProjectInfo": ["createProjectInfo"]
        },

        postMixInProperties: function () {
            this.inherited(arguments);

            if (!this.model) {
                this.own(
                    this.model = this.commandProvider = new ProjectComponentViewModel()
                );
            }

            // Get the current content language from the profile
            var self = this;
            this._profile = this._profile || dependency.resolve("epi.shell.Profile");
            when(this._profile.getContentLanguage(), function(contentLanguage){
                self._currentContentLanguage = contentLanguage;
            });
        },

        buildRendering: function() {
            this.inherited(arguments);

            this._setupList();
            this._setupProjectMenu();
            this._setupNewProjectButton();
            this._setupDnd();

        },

        startup: function () {

            if (this._started) { return; }
            this.inherited(arguments);

            var model = this.model;

            // TODO: Suggest we use the model throughout the whole gadget instead of
            // just setting the stores, as it would simplify the initialization and 
            // handling of the selectedProject
            this.projectSelector.set("store", model.projectStore);

            this.own(
                this.projectSelector.watch("value", lang.hitch(this, function(name, oldvalue, value) {
                    if (this.projectSelector.compare(value, this.model.selectedProject) !== 0) {
                        this.model.set("selectedProject", value);
                    }

                    this.model.refreshProject();
                }),
                this.watch("_placeholderStatus", lang.hitch(this, "_handlePlaceHolderStates")),
                this.model.watch("hasProject", lang.hitch(this, "_setProjectStates" )),
                this.model.watch("totalProjectItems", lang.hitch(this, "_setProjectItemStates" ))
            ));

            this._list.startup();
            this.model.refreshProject();
        },

        layout: function () {
            this.inherited(arguments);

            this._resizeList();
        },

        _setProjectStates: function (name, oldvalue, value) {
            if (this.model.selectedProject && value && this.model.get("totalProjectItems") > 0) {
                // should set _placeholderStatus to 'ready' when there are projects available, projectitems, and a project is selected
                this.set("_placeholderStatus", "ready");
            } else if (this.model.selectedProject == null && value) {
                // should set _placeholderStatus to 'noSelectedProject' when there are projects available, no projectitems, and no project is selected
                this.set("_placeholderStatus", "noSelectedProject");
            } else if (value && this.model.get("totalProjectItems") === 0) {
                // should set _placeholderStatus to 'noProjectItems' when there are projects available, no projectitems, and a project is selected
                this.set("_placeholderStatus", "noProjectItems");
            } else {
                // should set _placeholderStatus to 'noProjects' when there are no projects, no projectitems, and no project is selected
                this.set("_placeholderStatus", "noProjects");
            }
        },

        _setProjectItemStates: function(name, oldvalue, value) {
            if (value > 0) {
                // should set _placeholderStatus to 'ready' when there are projects available, projectitems, and a project is selected
                this.set("_placeholderStatus", "ready");
            } else if (this.model.get("hasProject")) {
                // should set _placeholderStatus to 'noProjectItems' when there are projects available, no projectitems, and a project is selected
                this.set("_placeholderStatus", "noProjectItems");
            }
        },

        _handlePlaceHolderStates: function (name, oldvalue, value) {

            // Toggle component elements visibility based on current _placeholderStatus
            switch (value) {
                case "noSelectedProject":
                    this.model.set("createProjectInfo", res.placeholder.noprojectselected);

                    domClass.toggle(this.projectSelector.domNode, "dijitHidden", false);
                    domClass.toggle(this.publishContainer.domNode, "dijitHidden", false);
                    domClass.toggle(this.placeholderNode, "dijitHidden", false);
                    domClass.toggle(this.itemList, "dijitHidden", true);
                    domClass.toggle(this.overlayNode, "projectVisible", false);
                    
                    var sort = this.projectSelector.dropDown.grid.get("sort");
                    this.model.selectNextProject(sort);

                    break;
                case "noProjects":
                    this.model.set("createProjectInfo", res.placeholder.noprojectscreated);

                    domClass.toggle(this.projectSelector.domNode, "dijitHidden", true);
                    domClass.toggle(this.publishContainer.domNode, "dijitHidden", true);
                    domClass.toggle(this.placeholderNode, "dijitHidden", false);
                    domClass.toggle(this.itemList, "dijitHidden", true);
                    domClass.toggle(this.overlayNode, "projectVisible", false);

                    break;
                case "noProjectItems":
                    domClass.toggle(this.projectSelector.domNode, "dijitHidden", false);
                    domClass.toggle(this.publishContainer.domNode, "dijitHidden", false);
                    domClass.toggle(this.placeholderNode, "dijitHidden", true);
                    domClass.toggle(this.itemList, "dijitHidden", false);
                    domClass.toggle(this.overlayNode, "projectVisible", true);

                    break;
                default:
                    domClass.toggle(this.projectSelector.domNode, "dijitHidden", false);
                    domClass.toggle(this.publishContainer.domNode, "dijitHidden", false);
                    domClass.toggle(this.placeholderNode, "dijitHidden", true);
                    domClass.toggle(this.itemList, "dijitHidden", false);
                    domClass.toggle(this.overlayNode, "projectVisible", false);

            }

        },

        _setupList: function () {

            var currentContentLanguage = this._currentContentLanguage,
                accessLevel = ContentActionSupport.accessLevel,
                projectSelector = this.projectSelector;

            var contextMenu = new ContextMenu({ category: "itemContext" }), // Owned by the list
                list = new this._listClass({
                    formatters: [
                        formatters.contentItemFactory(
                            "name", // nameSelector
                            function(item) {
                                if (ContentActionSupport.hasAccess(item.accessMask, accessLevel.Read) && item.path) {
                                    return item.path.join(" > ");
                                }

                                return item.name;
                            }, // titleSelector
                            "typeIdentifier", // iconSelector
                            { hasMenu: true, settings: { title: sharedResources.action.options } }, // contextMenu settings
                            null, // thumbnailSelector
                            function(value) { //languageSelector
                                // If the current content language differs from the contentLanguage for the project item
                                // return the content language for the project item
                                if(value.contentLanguage !== currentContentLanguage) {
                                    return { language: value.contentLanguage};
                                } {
                                    return null;
                                }
                            }
                            ),
                        this.model.statusFormatter // Adds status icon
                    ],
                    contextMenu: contextMenu,
                    deselectOnRefresh: true,
                    selectionMode: "single",
                    selectionEvents: "click,dgrid-cellfocusin"
                }, this.itemList);

            this.own(
                this._list = list,
                // listen to view model for request to refresh the list
                this.model.on("refresh", function() {
                    projectSelector.refresh();
                    list.refresh({ keepScrollPosition: true });
                }),
                list.on("dgrid-refresh-complete", lang.hitch(this, function(evt) {
                    when(evt.results.total, lang.hitch(this, function(total) {
                        this.model.set("totalProjectItems", total);
                    }));
                })),
                list.on("dgrid-select", lang.hitch(this, function(evt) {
                    var item = evt.rows[0].data;
                    this.model.set("selectedProjectItem", item);
                })),
                list.on(".dgrid-row:dblclick", lang.hitch(this, function(evt) {
                    var item = this._list.row(evt).data;
                    if(ContentActionSupport.hasAccess(item.accessMask, accessLevel.Read)) {
                        this.model.requestContextChange(item.contentLink);
                    }
                })),
                list.on(on.selector(".dgrid-row", events.keys.del), lang.hitch(this, function () {
                    this.model.namedCommands.removeProjectItem.execute();
                }))
            );

            contextMenu.addProvider(this.model);
        },

        _resizeList: function() {
            var toolbarSize = domGeometry.getMarginBox(this.toolbar.domNode),
                size = lang.mixin({}, this._contentBox, { h: this._contentBox.h - toolbarSize.h });

            domGeometry.setMarginBox(this.itemList, size);
            this._list.resize();

            domGeometry.setMarginBox(this.placeholderNode, {
                l: 0,
                t: toolbarSize.h,
                h: size.h,
                w: size.w
            });
            domGeometry.setMarginBox(this.overlayNode, {
                l: 0,
                t: toolbarSize.h,
                h: size.h,
                w: size.w
            });

        },

        _setupProjectMenu: function(){

            var configuration = [
                    {
                        builder: new ButtonBuilder({ settings: { "class": "epi-fullWidthButton Sweet", isExecutingClass: "epi-loading" } }),
                        category: "publishmenu",
                        target: this.publishMenuNode
                    },
                    {
                        builder: new ButtonBuilder({ settings: { "class": "epi-flat epi-chromeless epi-chromeless--icon-only", showLabel: false } }),
                        category: "projectButton",
                        target: this.toolbarGroupNode
                    }
                ];

            this.own(
                new MenuAssembler({
                    configuration: configuration,
                    commandSource: this.model
                }),
                on(this.publishMenuNode, "click", lang.hitch(this, function() {
                    // Return focus back to the drop down button when the publish button is clicked. Otherwise
                    // focus will be reset back to the window when the publish button is hidden.
                    this.publishContainer.focus();
                }))
            );
        },

        _setupNewProjectButton: function() {

            this._newProjectButton = new Button({
                "class": "epi-flat epi-chromeless",
                iconClass: "epi-iconPlus",
                label: res.command.newproject.label,
                onClick: lang.hitch(this, this._newProjectButtonClickHandler)
            }, this.createProjectButtonNode);

        },

        _setupDnd: function() {
            var model = this.model;

            this.own(
                this._dndTarget = new Target(this.overlayNode, {
                    createItemOnDrop: false,
                    accept: ["episerver.core.icontentdata.reference"]
                }),
                aspect.after(this._dndTarget, "onDropData", function(dndItems) {
                    var items = dndItems.map(function(item) {
                        return item.data;
                    });
                    model.addProjectItems(items);
                }, true)
            );
        },

        _publishMenuClickHandler: function() {
            this.model.refreshProject();
        },


        _newProjectButtonClickHandler: function () {
            this.model.namedCommands.addProject.execute();
        },

        _setCreatedAttr: { node: "createdNode", type: "innerText" },

        _setCreatedByAttr: { node: "createdByNode", type: "innerText" },

        _setCreateProjectInfoAttr: { node: "createProjectInfoNode", type: "innerText" },

        _setProjectItemQueryAttr: function (value) {
            // Set store to null if query is null to stop list updates
            var store = value ? this.model.projectItemStore : null;
            this._list.set("store", store, value);
        },

        _setProjectItemSortOrderAttr: function(sortOrder) {
            if (sortOrder && sortOrder !== this.projectItemSortOrder) {
                this._set("projectItemSortOrder", sortOrder);
                var sorting = [{attribute: sortOrder.substring(1), descending: sortOrder.charAt(0) === "-"}];
                // Set Name as secondary sort attribute unless Name is the primary
                if (sorting[0].attribute !== "Name") {
                    sorting.push({attribute: "Name"});
                }
                this._list.set("sort", sorting);
            }
        },

        _setSelectedProjectAttr: function (value) {
            var projectSelected = !!value;
            this._setProjectSelectorValue(value);
            this.publishContainer.set("disabled", !projectSelected);

            // Don't allow the drag and drop overlay to show if there is no project selected or if the selected project is published.
            domClass.toggle(this.overlayNode, "dijitHidden", !projectSelected || value && value.status === "Published");

            // set the publish fail message & notification bar depending on project status
            this._setPublishFailState(value);
            this._setProjectStatusMessage(value);
        },

        _setProjectSelectorValue: function (value) {
            //Prevent loop
            if (this.projectSelector.compare(value, this.projectSelector.value) !== 0) {
                this.projectSelector.set("value", value);
            }
        },

        _setPublishFailState: function (selectedProject) {

            // clear all the UI state
            domConstruct.empty(this.publishFailNotificationText);
            domClass.add(this.publishFailNotificationBar, "dijitHidden");

            if (selectedProject && selectedProject.status === "PublishFailed") {

                // set the notification bar
                var itemOrItems = selectedProject.failedItemsCount > 1 ? res.notifications.publishfailed.notificationtextmultipleitems : res.notifications.publishfailed.notificationtextsingleitem;

                var text = lang.replace(itemOrItems, { failedItemsCount: selectedProject.failedItemsCount });

                domClass.remove(this.publishFailNotificationBar, "dijitHidden");
                this.publishFailNotificationText.textContent = text;

            }
        },

        _setProjectStatusMessage: function (selectedProject) {

            if (selectedProject) {

                var message;
                switch (selectedProject.status) {
                    case "Active":
                    case "PublishFailed":
                        message = res.status.message.itemsmustbereadytopublish;
                        break;
                    case "Published":
                        message = res.status.message.hasbeenpublished;
                        break;
                    case "Publishing":
                        message = res.status.message.publishing;
                        break;
                    default:
                        message = "";
                        break;
                }

                domClass.toggle(this.projectStatusMessage, "dijitHidden", !message);

                if (message !== "") {
                    this.projectStatusMessage.textContent = message;
                }
            }
        }

    });
});
